searchData={"items":[{"type":"module","title":"FancyFences","doc":"Post-process code blocks in your markdown docs.\n\n`FancyFences` is a tiny wrapper around `ExDoc.Markdown.Earmark` that post processes\nfenced markdown code blocks. You can use it to:\n\n- Ensure that the code examples are up to date with your code.\n- Automatically inject the output of the code block.\n- Enrich your docs with VegaLite plots or mermaid charts.\n- Use it in a more hacky way when interpolation is not possible.","ref":"FancyFences.html"},{"type":"module","title":"Usage - FancyFences","doc":"In order to use `FancyFences` you need to set the `:markdown_processor` option\nin the `:docs` section as following:\n\n    docs: [\n      markdown_processor: {FancyFences, [fences: processors]}\n    ]\n\nwhere `processors` is expected to be a map with all processors that will be\napplied on code blocks. The keys of the `proecessors` map are the code\n_language_ on which this specific processor will be applied. The values of the\n`processors` map are expected to be `mfa` tuples with the function that will be\napplied. For example the following configuration:\n\n    docs: [\n      markdown_processor: {FancyFences, [fences: fancy_processors()]}\n    ]\n\n    defp fancy_processors do\n      %{\n        \"elixir\" => {FancyFences.Processors, :format_code, []},\n        \"elixir::inspect\" => FancyFences.Processors, :inspect_code, [])\n      }\n    end\n\nwill apply two processors:\n\n- Each `elixir` code block will be post-processed using the `FancyFences.Processors.format_code/1`\nwhich will format the inline code.\n- Each `elixir::inspect` code block will be post-processed using the `FancyFences.Processors.inspect_code/2`\nwhich will replace the existing code block with two blocks:\n  - An `:elixir` code block including the initial code\n  - A second `:elixir` block with the output of the evaluation of the first block.\n\nThis is useful for evaluating during docs generation fenced code blocks, without having to\nmanually write the expected output, also ensuring that your docs are always up to date, since\nif the underlying functions change then `mix docs` will fail.\n\n> ##","ref":"FancyFences.html#module-usage"},{"type":"module","title":"Recursive application of fence processors {: .tip} - FancyFences","doc":">\n> Norice that fence processors are applied recursively. This means that in the previous\n> example the `elixir` injected code blocks will also be processed by the format\n> processor, ensuring that both the raw code input and the evaluation output will be\n> properly formatted.","ref":"FancyFences.html#module-recursive-application-of-fence-processors-tip"},{"type":"module","title":"Fence processors - FancyFences","doc":"Fence processors are simple elixir functions that expect as input a string corresponding to\nthe code in the markdown code block and an optional keyword list and return a markdown that\nwill be injected in the docs.\n\n#","ref":"FancyFences.html#module-fence-processors"},{"type":"module","title":"Writing a custom processor - FancyFences","doc":"Let's see a real world example, how [`Tucan`](https://hexdocs.pm/tucan/Tucan.html) uses\n`FancyFences` in order to render both the sample code and the corresponding `vega-lite`\nspecification.\n\n\n`Tucan` defines a fence processor that it expects a code blocks that returns a `VegaLite`\nstruct and it generates two code blocks:\n\n- The original code formatted.\n- The `json` `vega-lite` specification of the `VegaLite` struct.\n\n~~~elixir\ndef tucan(code, _opts \\\\ []) do\n  {%VegaLite{} = plot, _} = Code.eval_string(code, [], __ENV__)\n\n  spec = VegaLite.to_spec(plot)\n\n  \"\"\"\n  ```elixir\n  #{Code.format_string!(code)}\n  ```\n\n  ```vega-lite\n  #{Jason.encode!(spec)}\n  ```\n  \"\"\"\nend\n~~~\n\nIn order to enable it you only need to configure to which markdown code _language_ this\nwill be applied. In the `Tucan` case it is applied on code blocks marked as `tucan`:\n\n    docs: [\n      markdown_processor:\n        {FancyFences,\n         [\n           fences: %{\n             \"tucan\" => {Tucan.Docs, :tucan, []}\n           }\n         ]},\n    ]","ref":"FancyFences.html#module-writing-a-custom-processor"},{"type":"module","title":"FancyFences.Processors","doc":"Common fence processors.","ref":"FancyFences.Processors.html"},{"type":"function","title":"FancyFences.Processors.fence_processor_doc/1","doc":"A fence processor for documenting fence processors usage.\n\nThis can be used for documenting fence processors. It return an admonition\nblock containing both the sample code and the output of the fence processor.","ref":"FancyFences.Processors.html#fence_processor_doc/1"},{"type":"function","title":"Usage - FancyFences.Processors.fence_processor_doc/1","doc":"It expects a map with the following key-value pairs:\n\n- `block` - the code block to use as an example\n- `processor` - an anonymous function with the processor that will be used.\n\nFor example:\n\n~~~\n```fence-processor\n%{\n  block: \"Enum.map([1, 2, 3], fn x -> 2*x end)\",\n  processor: fn block -> FancyFences.Processors.inspect_code(block) end\n}\n```\n~~~\n\n```fence-processor\n%{\n  block: \"\"\"\n  %{\n     block: \"1 + 1\",\n     processor: fn block -> \n       \"**Code:** `\"<> block <> \"`\"\n     end\n  }\n  \"\"\",\n  processor: fn block -> FancyFences.Processors.fence_processor_doc(block) end\n}\n```\n\nNotice that above we used `fence_processor_doc/1` to document itself, that's why\nwe have the nested admonition block.","ref":"FancyFences.Processors.html#fence_processor_doc/1-usage"},{"type":"function","title":"FancyFences.Processors.format_code/1","doc":"Formats the given code block.\n\n```fence-processor\n%{\n  block: \"\"\"\n  for x <- [1, 2, 3] do\n  2 * x\n  end\n  \"\"\",\n  processor: fn block -> FancyFences.Processors.format_code(block) end\n}\n```","ref":"FancyFences.Processors.html#format_code/1"},{"type":"function","title":"FancyFences.Processors.inspect_code/2","doc":"Embeds the original code and the evaluated result.","ref":"FancyFences.Processors.html#inspect_code/2"},{"type":"function","title":"Options - FancyFences.Processors.inspect_code/2","doc":"* `:format` (`boolean`) - If set to `true` the code blocks will\n  be formatted before inection. Defaults to `false`.\n\n```fence-processor\n%{\n  block: \"Enum.map([1, 2, 3, 4], fn x -> 2*x end)\",\n  processor: fn block -> FancyFences.Processors.inspect_code(block) end\n}\n```","ref":"FancyFences.Processors.html#inspect_code/2-options"},{"type":"function","title":"FancyFences.Processors.multi_inspect/2","doc":"Embeds the original code and the evaluated results supporing multiple inspection points.","ref":"FancyFences.Processors.html#multi_inspect/2"},{"type":"function","title":"Options - FancyFences.Processors.multi_inspect/2","doc":"* `:format` (`boolean`) - If set to `true` the code blocks will\n  be formatted before inection. Defaults to `false`.\n* `:iex_prefix` (`boolean`) - If set to `true` an `iex>` prefix will be added to\nthe original code lines similar to doc tests. Defaults to `false`.\n* `:separator` (`string`) - The separator for inspection points. The original code\nwill be split on these separators and an inspect statement will be added after\neach one. Defaults to `>>>`.\n\n```fence-processor\n%{\n  block: \"\"\"\n  list = [1, 2, 3, 4]\n  Enum.map(list, fn x -> 2*x end)\n  >>>\n\n  Enum.map(list, fn x -> 3 + x end)\n  >>>\n\n  Enum.reduce(list, 0, fn x, acc -> x + acc end)\n  \"\"\",\n  processor: fn block -> FancyFences.Processors.multi_inspect(block, format: true, iex_prefix: true) end\n}\n```\n\nNotice that the last statement is inspected by default and there is no need to add\na separator.","ref":"FancyFences.Processors.html#multi_inspect/2-options"},{"type":"extras","title":"Overview","doc":"# FancyFences\n\n[![Actions Status](https://github.com/pnezis/fancy_fences/actions/workflows/elixir.yml/badge.svg)](https://github.com/pnezis/fancy_fences/actions)\n[![Package](https://img.shields.io/badge/-Package-important)](https://hex.pm/packages/fancy_fences)\n[![Documentation](https://img.shields.io/badge/-Documentation-blueviolet)](https://hexdocs.pm/fancy_fences)\n\n`FancyFences` is a markdown processor on top of [`EarmarkParser`](https://github.com/pragdave/earmark)\n(the default markdown processor used by [`ExDoc`](https://github.com/elixir-lang/ex_doc).\nYou can use it to conditionally post-process code blocks allowing you to:\n\n- Ensure that the code examples are valid\n- Format code blocks\n- Evaluate code blocks and include the output in the documenation, for example\nyou can:\n  - add the `inspect` ouptut of a code block in order to have up to date code\n  samples in your docs\n  - auto-generate vega-lite or mermaid plots\n  - use it instead of interplation for evaluating functions within the current module.\n\n![mermaid example](https://github.com/pnezis/fancy_fences/raw/main/assets/mermaid.png)","ref":"readme.html"},{"type":"extras","title":"Usage - Overview","doc":"In order to use `FancyFences` you need to set the `:markdown_processor` option\nin the `:docs` section as following:\n\n```elixir\ndocs: [\n  markdown_processor: {FancyFences, [fences: processors]}\n]\n```\n\nwhere `processors` defines the code blocks processors.\n\n```elixir\ndocs: [\n  markdown_processor: {FancyFences, [fences: fancy_processors()]}\n]\n\ndefp fancy_processors do\n  %{\n    \"format\" => {FancyFences.Processors, :format_code, []},\n    \"inspect\" => {FancyFences.Processors, :inspect_code, [format: true]},\n    \"vl\" => {MyProcessors, :vega_lite, []},\n    \"mermaid\" => {MyProcessors, :mermaid, []}\n  }\nend\n```\n\nwill apply the following processors:\n\n- Each `format` code block will be post-processed using the `FancyFences.Processors.format_code/1`\nwhich will format the inline code.\n- Each `inspect` code block will be post-processed using the `FancyFences.Processors.inspect_code/2`\nwhich will replace the existing code block with two blocks:\n  - An `:elixir` code block including the initial code\n  - A second `:elixir` block with the output of the evaluation of the first block.\n- Each `vl` block will apply a custom processor that evaluates the inline code\nblock and replaces it with the original block and the evaluated vega-lite spec.\nYou can see such a processor used throughout [`Tucan` docs](https://hexdocs.pm/tucan/Tucan.html)\n- Similarly for the mermaid blocks.\n\nNow in your markdown docs you can use the above processors as language and `fancy_fences`\nwill apply the required transformations during `mix docs` invocation.\n\n#","ref":"readme.html#usage"},{"type":"extras","title":"Examples - Overview","doc":"`inspect` fence processor example:\n\n![inspect example](https://github.com/pnezis/fancy_fences/raw/main/assets/inspect.png)\n\n`vl` fence processor example:\n\n![vega-lite example](https://github.com/pnezis/fancy_fences/raw/main/assets/vl.png)\n\n#","ref":"readme.html#examples"},{"type":"extras","title":"Example project - Overview","doc":"You can find a sample project [using `fancy_fences` here](https://github.com/pnezis/fancy_fences_example).","ref":"readme.html#example-project"},{"type":"extras","title":"Installation - Overview","doc":"In order to install the package add the following to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:fancy_fences, \"~> 0.3.0\", only: :dev, runtime: false}\n  ]\nend\n```\n\nand configure your fence processors accoring to the [docs](https://hexdocs.pm/fancy_fences).","ref":"readme.html#installation"},{"type":"extras","title":"License - Overview","doc":"Copyright (c) 2023 Panagiotis Nezis\n\n`fancy_fences` is released under the MIT License. See the [LICENSE](LICENSE) file\nfor more details.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n## [v0.3.1](https://github.com/pnezis/fancy_fences/tree/v0.3.1) (2024-04-07)\n\n#","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Add `multi_inspect` processor\n\n## [v0.3.0](https://github.com/pnezis/fancy_fences/tree/v0.3.0) (2023-10-02)\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Add some default fence processors under `FancyFences.Processors`\n- Update documentation\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Apply fence processors recursively under any HTML AST element.\n\n## [v0.2.0](https://github.com/pnezis/fancy_fences/tree/v0.2.0) (2023-08-09)\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Support fancy fences in blockquotes\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"## [v0.1.0](https://github.com/pnezis/fancy_fences/tree/v0.1.0) (2023-06-20)\n\nInitial release.","ref":"changelog.html#fixed"},{"type":"extras","title":"LICENSE","doc":"# MIT License\n\nCopyright (c) 2023 Panagiotis Nezis\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}